# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: region/v1beta1/region.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x1bregion/v1beta1/region.proto\x12\x0eregion.v1beta1\x1a\x1fgoogle/protobuf/timestamp.proto"\xd8\x01\n\nRegionInfo\x12"\n\x02id\x18\x01 \x01(\x0e\x32\x16.region.v1beta1.Region\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x0f\n\x07\x63ountry\x18\x03 \x01(\t\x12\x0c\n\x04zone\x18\x04 \x01(\t\x12\r\n\x05\x63loud\x18\x05 \x01(\t\x12\x0f\n\x07\x63luster\x18\x06 \x01(\t\x12,\n\x08modified\x18\x0e \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12+\n\x07\x63reated\x18\x0f \x01(\x0b\x32\x1a.google.protobuf.Timestamp*\xdb\x1a\n\x06Region\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x16\n\x12STG_LKE_US_EAST_1A\x10\x01\x12\x14\n\x0eLKE_AP_WEST_1A\x10\xe0\xe1R\x12\x17\n\x11LKE_CA_CENTRAL_1A\x10\xaa\xcd\x44\x12\x19\n\x13LKE_AP_SOUTHEAST_1A\x10\xf4\x9d?\x12\x17\n\x11LKE_US_CENTRAL_1A\x10\x9e\xa7p\x12\x14\n\x0eLKE_US_WEST_1A\x10\xa8\xa7p\x12\x19\n\x13LKE_US_SOUTHEAST_1A\x10\xb2\xa7p\x12\x14\n\x0eLKE_US_EAST_1A\x10\xbc\xa7p\x12\x14\n\x0eLKE_EU_WEST_1A\x10\x96\xbao\x12\x15\n\x0fLKE_AP_SOUTH_1A\x10\xc0\xf1g\x12\x17\n\x11LKE_EU_CENTRAL_1A\x10\xba\xf1M\x12\x19\n\x13LKE_AP_NORTHEAST_1A\x10\xe4\xfbT\x12\x17\n\x10GCP_ASIA_EAST_1A\x10\xb1\xdb\x83\x01\x12\x17\n\x10GCP_ASIA_EAST_1B\x10\xb2\xdb\x83\x01\x12\x17\n\x10GCP_ASIA_EAST_1C\x10\xb3\xdb\x83\x01\x12\x17\n\x10GCP_ASIA_EAST_2A\x10\xca\x88\x8f\x01\x12\x17\n\x10GCP_ASIA_EAST_2B\x10\xcb\x88\x8f\x01\x12\x17\n\x10GCP_ASIA_EAST_2C\x10\xcc\x88\x8f\x01\x12\x1c\n\x15GCP_ASIA_NORTHEAST_1A\x10\xd4\xff\x91\x01\x12\x1c\n\x15GCP_ASIA_NORTHEAST_1B\x10\xd5\xff\x91\x01\x12\x1c\n\x15GCP_ASIA_NORTHEAST_1C\x10\xd6\xff\x91\x01\x12\x1c\n\x15GCP_ASIA_NORTHEAST_2A\x10\xde\xff\x91\x01\x12\x1c\n\x15GCP_ASIA_NORTHEAST_2B\x10\xdf\xff\x91\x01\x12\x1c\n\x15GCP_ASIA_NORTHEAST_2C\x10\xe0\xff\x91\x01\x12\x1c\n\x15GCP_ASIA_NORTHEAST_3A\x10\xb8\x8c\x93\x01\x12\x1c\n\x15GCP_ASIA_NORTHEAST_3B\x10\xb9\x8c\x93\x01\x12\x1c\n\x15GCP_ASIA_NORTHEAST_3C\x10\xba\x8c\x93\x01\x12\x18\n\x11GCP_ASIA_SOUTH_1A\x10\xd3\xe6\x8f\x01\x12\x18\n\x11GCP_ASIA_SOUTH_1B\x10\xd4\xe6\x8f\x01\x12\x18\n\x11GCP_ASIA_SOUTH_1C\x10\xd5\xe6\x8f\x01\x12\x18\n\x11GCP_ASIA_SOUTH_2A\x10\xdd\xe6\x8f\x01\x12\x18\n\x11GCP_ASIA_SOUTH_2B\x10\xde\xe6\x8f\x01\x12\x18\n\x11GCP_ASIA_SOUTH_2C\x10\xdf\xe6\x8f\x01\x12\x1c\n\x15GCP_ASIA_SOUTHEAST_1A\x10\xf6\xf5\xa4\x01\x12\x1c\n\x15GCP_ASIA_SOUTHEAST_1B\x10\xf7\xf5\xa4\x01\x12\x1c\n\x15GCP_ASIA_SOUTHEAST_1C\x10\xf8\xf5\xa4\x01\x12\x1c\n\x15GCP_ASIA_SOUTHEAST_2A\x10\x90\x86\x90\x01\x12\x1c\n\x15GCP_ASIA_SOUTHEAST_2B\x10\x91\x86\x90\x01\x12\x1c\n\x15GCP_ASIA_SOUTHEAST_2C\x10\x92\x86\x90\x01\x12 \n\x1aGCP_AUSTRALIA_SOUTHEAST_1A\x10\xfa\xa2|\x12 \n\x1aGCP_AUSTRALIA_SOUTHEAST_1B\x10\xfb\xa2|\x12 \n\x1aGCP_AUSTRALIA_SOUTHEAST_1C\x10\xfc\xa2|\x12 \n\x1aGCP_AUSTRALIA_SOUTHEAST_2A\x10\x84\xa3|\x12 \n\x1aGCP_AUSTRALIA_SOUTHEAST_2B\x10\x85\xa3|\x12 \n\x1aGCP_AUSTRALIA_SOUTHEAST_2C\x10\x86\xa3|\x12\x1c\n\x15GCP_EUROPE_CENTRAL_2A\x10\xae\xd6\x9f\x01\x12\x1c\n\x15GCP_EUROPE_CENTRAL_2B\x10\xaf\xd6\x9f\x01\x12\x1c\n\x15GCP_EUROPE_CENTRAL_2C\x10\xb0\xd6\x9f\x01\x12\x1a\n\x13GCP_EUROPE_NORTH_1A\x10\xe8\x8b\x89\x01\x12\x1a\n\x13GCP_EUROPE_NORTH_1B\x10\xe9\x8b\x89\x01\x12\x1a\n\x13GCP_EUROPE_NORTH_1C\x10\xea\x8b\x89\x01\x12\x1e\n\x17GCP_EUROPE_SOUTHWEST_1A\x10\xa2\xa2\xa6\x01\x12\x1e\n\x17GCP_EUROPE_SOUTHWEST_1B\x10\xa3\xa2\xa6\x01\x12\x1e\n\x17GCP_EUROPE_SOUTHWEST_1C\x10\xa4\xa2\xa6\x01\x12\x18\n\x12GCP_EUROPE_WEST_1B\x10\xcd\xbf}\x12\x18\n\x12GCP_EUROPE_WEST_1C\x10\xce\xbf}\x12\x18\n\x12GCP_EUROPE_WEST_1D\x10\xcf\xbf}\x12\x1a\n\x13GCP_EUROPE_WEST_12A\x10\xf6\xa2\x91\x01\x12\x1a\n\x13GCP_EUROPE_WEST_12B\x10\xf7\xa2\x91\x01\x12\x1a\n\x13GCP_EUROPE_WEST_12C\x10\xf8\xa2\x91\x01\x12\x19\n\x12GCP_EUROPE_WEST_2A\x10\xb0\xbf\xac\x01\x12\x19\n\x12GCP_EUROPE_WEST_2B\x10\xb1\xbf\xac\x01\x12\x19\n\x12GCP_EUROPE_WEST_2C\x10\xb2\xbf\xac\x01\x12\x19\n\x12GCP_EUROPE_WEST_3A\x10\xca\xf6\x8a\x01\x12\x19\n\x12GCP_EUROPE_WEST_3B\x10\xcb\xf6\x8a\x01\x12\x19\n\x12GCP_EUROPE_WEST_3C\x10\xcc\xf6\x8a\x01\x12\x19\n\x12GCP_EUROPE_WEST_4A\x10\xb4\xa7\x9a\x01\x12\x19\n\x12GCP_EUROPE_WEST_4B\x10\xb5\xa7\x9a\x01\x12\x19\n\x12GCP_EUROPE_WEST_4C\x10\xb6\xa7\x9a\x01\x12\x19\n\x12GCP_EUROPE_WEST_6A\x10\xde\x9c\xa8\x01\x12\x19\n\x12GCP_EUROPE_WEST_6B\x10\xdf\x9c\xa8\x01\x12\x19\n\x12GCP_EUROPE_WEST_6C\x10\xe0\x9c\xa8\x01\x12\x19\n\x12GCP_EUROPE_WEST_8A\x10\xa8\xa3\x91\x01\x12\x19\n\x12GCP_EUROPE_WEST_8B\x10\xa9\xa3\x91\x01\x12\x19\n\x12GCP_EUROPE_WEST_8C\x10\xaa\xa3\x91\x01\x12\x19\n\x12GCP_EUROPE_WEST_9A\x10\xe2\xab\x89\x01\x12\x19\n\x12GCP_EUROPE_WEST_9B\x10\xe3\xab\x89\x01\x12\x19\n\x12GCP_EUROPE_WEST_9C\x10\xe4\xab\x89\x01\x12\x18\n\x11GCP_ME_CENTRAL_1A\x10\xbc\xa6\x90\x01\x12\x18\n\x11GCP_ME_CENTRAL_1B\x10\xbd\xa6\x90\x01\x12\x18\n\x11GCP_ME_CENTRAL_1C\x10\xbe\xa6\x90\x01\x12\x15\n\x0eGCP_ME_WEST_1A\x10\xa6\x84\x91\x01\x12\x15\n\x0eGCP_ME_WEST_1B\x10\xa7\x84\x91\x01\x12\x15\n\x0eGCP_ME_WEST_1C\x10\xa8\x84\x91\x01\x12$\n\x1dGCP_NORTHAMERICA_NORTHEAST_1A\x10\xd0\xd3\x81\x01\x12$\n\x1dGCP_NORTHAMERICA_NORTHEAST_1B\x10\xd1\xd3\x81\x01\x12$\n\x1dGCP_NORTHAMERICA_NORTHEAST_1C\x10\xd2\xd3\x81\x01\x12$\n\x1dGCP_NORTHAMERICA_NORTHEAST_2A\x10\xda\xd3\x81\x01\x12$\n\x1dGCP_NORTHAMERICA_NORTHEAST_2B\x10\xdb\xd3\x81\x01\x12$\n\x1dGCP_NORTHAMERICA_NORTHEAST_2C\x10\xdc\xd3\x81\x01\x12\x1e\n\x18GCP_SOUTHAMERICA_EAST_1A\x10\xe4\xdc~\x12\x1e\n\x18GCP_SOUTHAMERICA_EAST_1B\x10\xe5\xdc~\x12\x1e\n\x18GCP_SOUTHAMERICA_EAST_1C\x10\xe6\xdc~\x12\x1f\n\x18GCP_SOUTHAMERICA_WEST_1A\x10\xce\xae\x83\x01\x12\x1f\n\x18GCP_SOUTHAMERICA_WEST_1B\x10\xcf\xae\x83\x01\x12\x1f\n\x18GCP_SOUTHAMERICA_WEST_1C\x10\xd0\xae\x83\x01\x12\x18\n\x11GCP_US_CENTRAL_1A\x10\xd8\xad\xad\x01\x12\x18\n\x11GCP_US_CENTRAL_1B\x10\xd9\xad\xad\x01\x12\x18\n\x11GCP_US_CENTRAL_1C\x10\xda\xad\xad\x01\x12\x18\n\x11GCP_US_CENTRAL_1F\x10\xdd\xad\xad\x01\x12\x15\n\x0eGCP_US_EAST_1B\x10\xe3\xad\xad\x01\x12\x15\n\x0eGCP_US_EAST_1C\x10\xe4\xad\xad\x01\x12\x15\n\x0eGCP_US_EAST_1D\x10\xe5\xad\xad\x01\x12\x15\n\x0eGCP_US_EAST_4A\x10\xec\xad\xad\x01\x12\x15\n\x0eGCP_US_EAST_4B\x10\xed\xad\xad\x01\x12\x15\n\x0eGCP_US_EAST_4C\x10\xee\xad\xad\x01\x12\x15\n\x0eGCP_US_EAST_5A\x10\xf6\xad\xad\x01\x12\x15\n\x0eGCP_US_EAST_5B\x10\xf7\xad\xad\x01\x12\x15\n\x0eGCP_US_EAST_5C\x10\xf8\xad\xad\x01\x12\x16\n\x0fGCP_US_SOUTH_1A\x10\x80\xae\xad\x01\x12\x16\n\x0fGCP_US_SOUTH_1B\x10\x81\xae\xad\x01\x12\x16\n\x0fGCP_US_SOUTH_1C\x10\x82\xae\xad\x01\x12\x15\n\x0eGCP_US_WEST_1A\x10\x8a\xae\xad\x01\x12\x15\n\x0eGCP_US_WEST_1B\x10\x8b\xae\xad\x01\x12\x15\n\x0eGCP_US_WEST_1C\x10\x8c\xae\xad\x01\x12\x15\n\x0eGCP_US_WEST_2A\x10\x94\xae\xad\x01\x12\x15\n\x0eGCP_US_WEST_2B\x10\x95\xae\xad\x01\x12\x15\n\x0eGCP_US_WEST_2C\x10\x96\xae\xad\x01\x12\x15\n\x0eGCP_US_WEST_3A\x10\x9e\xae\xad\x01\x12\x15\n\x0eGCP_US_WEST_3B\x10\x9f\xae\xad\x01\x12\x15\n\x0eGCP_US_WEST_3C\x10\xa0\xae\xad\x01\x12\x15\n\x0eGCP_US_WEST_4A\x10\xa8\xae\xad\x01\x12\x15\n\x0eGCP_US_WEST_4B\x10\xa9\xae\xad\x01\x12\x15\n\x0eGCP_US_WEST_4C\x10\xaa\xae\xad\x01\x62\x06proto3'
)

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(
    DESCRIPTOR, "region.v1beta1.region_pb2", globals()
)
if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    _REGION._serialized_start = 300
    _REGION._serialized_end = 3719
    _REGIONINFO._serialized_start = 81
    _REGIONINFO._serialized_end = 297
# @@protoc_insertion_point(module_scope)
