# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: api/v1beta1/groups.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x18\x61pi/v1beta1/groups.proto\x12\x0e\x65nsign.v1beta1\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1egoogle/protobuf/duration.proto\"\x91\x03\n\rConsumerGroup\x12\n\n\x02id\x18\x01 \x01(\x0c\x12\x12\n\nproject_id\x18\x02 \x01(\x0c\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x32\n\x08\x64\x65livery\x18\x04 \x01(\x0e\x32 .ensign.v1beta1.DeliverySemantic\x12\x33\n\x10\x64\x65livery_timeout\x18\x05 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x46\n\rtopic_offsets\x18\x0c \x03(\x0b\x32/.ensign.v1beta1.ConsumerGroup.TopicOffsetsEntry\x12\x11\n\tconsumers\x18\r \x03(\x0c\x12+\n\x07\x63reated\x18\x0e \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12,\n\x08modified\x18\x0f \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x1a\x33\n\x11TopicOffsetsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x04:\x02\x38\x01*Z\n\x10\x44\x65liverySemantic\x12\x0f\n\x0bUNSPECIFIED\x10\x00\x12\x10\n\x0c\x41T_MOST_ONCE\x10\x01\x12\x11\n\rAT_LEAST_ONCE\x10\x02\x12\x10\n\x0c\x45XACTLY_ONCE\x10\x03\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'api.v1beta1.groups_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _CONSUMERGROUP_TOPICOFFSETSENTRY._options = None
  _CONSUMERGROUP_TOPICOFFSETSENTRY._serialized_options = b'8\001'
  _DELIVERYSEMANTIC._serialized_start=513
  _DELIVERYSEMANTIC._serialized_end=603
  _CONSUMERGROUP._serialized_start=110
  _CONSUMERGROUP._serialized_end=511
  _CONSUMERGROUP_TOPICOFFSETSENTRY._serialized_start=460
  _CONSUMERGROUP_TOPICOFFSETSENTRY._serialized_end=511
# @@protoc_insertion_point(module_scope)
