# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: mimetype/v1beta1/mimetype.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1fmimetype/v1beta1/mimetype.proto\x12\x10mimetype.v1beta1*\xdc\x05\n\x04MIME\x12\x0f\n\x0bUNSPECIFIED\x10\x00\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x0e\n\nTEXT_PLAIN\x10\x01\x12\x0c\n\x08TEXT_CSV\x10\x02\x12\r\n\tTEXT_HTML\x10\x03\x12\x11\n\rTEXT_CALENDAR\x10\x04\x12\x1c\n\x18\x41PPLICATION_OCTET_STREAM\x10\x00\x12\x14\n\x10\x41PPLICATION_JSON\x10\x32\x12\x19\n\x15\x41PPLICATION_JSON_UTF8\x10\x32\x12\x17\n\x13\x41PPLICATION_JSON_LD\x10\x33\x12\x1a\n\x16\x41PPLICATION_JSON_LINES\x10\x34\x12\x16\n\x12\x41PPLICATION_UBJSON\x10\x35\x12\x14\n\x10\x41PPLICATION_BSON\x10\x36\x12\x13\n\x0f\x41PPLICATION_XML\x10\x64\x12\x14\n\x10\x41PPLICATION_ATOM\x10\x65\x12\x18\n\x13\x41PPLICATION_MSGPACK\x10\xfc\x01\x12\x18\n\x13\x41PPLICATION_PARQUET\x10\xfd\x01\x12\x15\n\x10\x41PPLICATION_AVRO\x10\xfe\x01\x12\x19\n\x14\x41PPLICATION_PROTOBUF\x10\xff\x01\x12\x14\n\x0f\x41PPLICATION_PDF\x10\x80\x04\x12\x1d\n\x18\x41PPLICATION_JAVA_ARCHIVE\x10\x81\x04\x12\x1e\n\x19\x41PPLICATION_PYTHON_PICKLE\x10\x82\x04\x12\x14\n\x0fUSER_SPECIFIED0\x10\xe8\x07\x12\x14\n\x0fUSER_SPECIFIED1\x10\xe9\x07\x12\x14\n\x0fUSER_SPECIFIED2\x10\xea\x07\x12\x14\n\x0fUSER_SPECIFIED3\x10\xeb\x07\x12\x14\n\x0fUSER_SPECIFIED4\x10\xec\x07\x12\x14\n\x0fUSER_SPECIFIED5\x10\xed\x07\x12\x14\n\x0fUSER_SPECIFIED6\x10\xee\x07\x12\x14\n\x0fUSER_SPECIFIED7\x10\xef\x07\x12\x14\n\x0fUSER_SPECIFIED8\x10\xf0\x07\x12\x14\n\x0fUSER_SPECIFIED9\x10\xf1\x07\x1a\x02\x10\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'mimetype.v1beta1.mimetype_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _MIME._options = None
  _MIME._serialized_options = b'\020\001'
  _MIME._serialized_start=54
  _MIME._serialized_end=786
# @@protoc_insertion_point(module_scope)
